/*
    Licensed Material - Property of IBM

    (C) Copyright 2015 IBM Corp.

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
*/


'use strict';

/*
 Allows for the creation of constants
 */
function define(name, value) {
    Object.defineProperty(exports, name, {
        value: value,
        enumerable: true
    });
}


define('LOG_PREFIX', 'create');
define('MFP', 'mfp');
define('DEVKIT_SERVER', 'MobileFoundation');
define('ICPA_SERVER', 'icpa-mobile-server');
define('PROTOCOL', 'http');
define('HOST', '127.0.0.1');
define('PORT', '9080');
define('USERNAME', 'admin');
define('PASSWORD', 'admin');
define('WORKLIGHT', 'worklight');
define('MFPCONFIG', 'mfpconfig');
define('CONF_SERVER_URL', '/mfp/api/az/v1/token');
define('CLI', 'mfpdev-cli');
define('CODE', 'code');
define('IONIC', 'ionic');
define('NATIVE', 'native');
define('ANDROID', 'android');
define('PWA', 'pwa');
define('IOS', 'ios');
define('PUSH', 'push');
define('PUSH_ANDROID_IMPORTS', `import com.ibm.mobilefirstplatform.clientsdk.android.push.api.MFPPush;
import com.ibm.mobilefirstplatform.clientsdk.android.push.api.MFPPushException;
import com.ibm.mobilefirstplatform.clientsdk.android.push.api.MFPPushNotificationListener;
import com.ibm.mobilefirstplatform.clientsdk.android.push.api.MFPPushResponseListener;
import com.ibm.mobilefirstplatform.clientsdk.android.push.api.MFPSimplePushNotification;\n`);
define('PUSH_ANDROID_DECL_CALL', `private MFPPush push;\n    `);
define('PUSH_ANDROID_IMPL_CALL', `     mfPushInit();\n   `);
define('PUSH_ANDROID_IMPL', `
    private void mfPushInit() {
        push = MFPPush.getInstance();
        push.initialize(this);
        push.registerDevice(null, new PushListener());
        push.listen(new PushListener());
    }  

    @Override
    public void onPause() {
        super.onPause();
        if (push != null) {
            push.hold();
        }
    }  

    @Override
    public void onResume() {
        super.onResume();
        if (push != null) {
            push.listen(new PushListener());
        }
    }\n`);
define('PUSH_ANDROID_LISTENERS', `
    class PushListener extends BaseListener implements MFPPushResponseListener<String>, MFPPushNotificationListener {
        @Override
        public void onSuccess(String s) {
            changeStatusMessage("Device successfully registered for push");
        }

        @Override
        public void onSuccess(org.json.JSONObject jobj) {
            changeStatusMessage("Device successfully registered for push");
        }

        @Override
        public void onFailure(MFPPushException e) {
            changeStatusMessage("Failed to register the device for push");
        }

        @Override
        public void onReceive(MFPSimplePushNotification mfpSimplePushNotification) {
            changeStatusMessage("Received notification : " + mfpSimplePushNotification.getAlert());
        }
    }\n`);   

define('PUSH_IOS_IMPORTS', `import IBMMobileFirstPlatformFoundationPush\n`);
define('PUSH_IOS_IMPL_CALL', `     mfPushInit()\n   `);  
define('PUSH_IOS_IMPL', `
    func mfPushInit() {
        let mfpPush = MFPPush.sharedInstance()
        mfpPush?.initialize()
        NotificationCenter.default.addObserver(self, selector: #selector(handlePush), name: Notification.Name("PushNotificationKey"), object: nil)
        mfpPush?.registerDevice(nil, completionHandler: pushHandler)
    }

    @objc func handlePush(_ notification: Notification) {
        if let msg = notification.userInfo?["message"] {
            changeStatusText("Received notification: \\(msg)")
        } else {
            changeStatusText(notification.userInfo?["errorMsg"] as! String)
        }   
    }`);  
define('PUSH_IOS_HANDLER', `  
    func pushHandler(response: WLResponse?, error: Error?) {
        if error == nil {
            changeStatusText("Device successfully registered for push")
        } else {
            changeStatusText("Failed to register the device for push")
        }    
    }\n`);
define('PUSH_IOS_DELEGATE_METHODS', `  
    func application(_ application: UIApplication, didRegisterForRemoteNotificationsWithDeviceToken deviceToken: Data) {
        MFPPush.sharedInstance()?.sendDeviceToken(deviceToken)
    }

    func application(_ application: UIApplication, didFailToRegisterForRemoteNotificationsWithError error: Error) {
        NotificationCenter.default.post(name: Notification.Name("PushNotificationKey"), object: nil, userInfo: ["errorMsg": error.localizedDescription])
    }    

    func application(_ application: UIApplication, didReceiveRemoteNotification userInfo: [AnyHashable: Any]) {
        showNotification(userInfo: userInfo)
    }
    
    func showNotification(userInfo: [AnyHashable: Any]) {
        if let notification = userInfo["aps"] as? NSDictionary,
            let alert = notification["alert"] as? NSDictionary,
            let body = alert["body"] as? String {
            NotificationCenter.default.post(name: Notification.Name("PushNotificationKey"), object: nil, userInfo: [ "message": body ])
        }    
    }`);    

define('ANALYTICS', 'analytics');
define('ANALYTICS_ANDROID_IMPORTS', `import com.worklight.common.WLAnalytics;
import com.worklight.common.WLAnalytics.DeviceEvent;
import com.worklight.common.Logger;
import com.worklight.common.Logger.LEVEL;\n`);
define('ANALYTICS_ANDROID_IMPL_CALL', `     mfAnalyticsInit();\n   `);
define('ANALYTICS_ANDROID_IMPL', `
    private void mfAnalyticsInit() {
        WLAnalytics.init(this.getApplication());
        WLAnalytics.addDeviceEventListener(DeviceEvent.LIFECYCLE);
        WLAnalytics.addDeviceEventListener(DeviceEvent.NETWORK);
        try {
            org.json.JSONObject json = new org.json.JSONObject();
            json.put("customDataKey", "customDataValue");
            WLAnalytics.log("App started successfully...", json);
            WLAnalytics.send();                               
        } catch (org.json.JSONException e) {
            e.printStackTrace();
        }
        Logger logger = Logger.getInstance("{appDetails.appName}");
        logger.setLevel(LEVEL.DEBUG);
        logger.debug("App started successfully...");
        logger.send();
    }\n`)
define('ANALYTICS_IOS_IMPL_CALL', `     mfAnalyticsInit();\n   `); 
define('ANALYTICS_IOS_IMPL', `
    func mfAnalyticsInit() {
        let mfAnalytics = WLAnalytics.sharedInstance()
        mfAnalytics?.addDeviceEventListener(LIFECYCLE)
        mfAnalytics?.addDeviceEventListener(NETWORK)
        mfAnalytics?.log("App started successfully...", withMetadata: ["customDataKey": "customDataValue"])
        mfAnalytics?.send()
        OCLogger.setLevel(OCLogger_DEBUG)
        let logger = OCLogger.getInstanceWithPackage("{appDetails.appName}")
        logger!.logDebugWithMessages(message:"App started successfully...")
        OCLogger.send()   
    }\n`);  
define('LIVEUPDATE', 'liveupdate');
define('LIVEUPDATE_ANDROID_IMPORTS', `import com.worklight.ibmmobilefirstplatformfoundationliveupdate.LiveUpdateManager;
import com.worklight.ibmmobilefirstplatformfoundationliveupdate.api.Configuration;
import com.worklight.ibmmobilefirstplatformfoundationliveupdate.api.ConfigurationListener;\n`);
define('LIVEUPDATE_ANDROID_IMPL_CALL', `     mfLiveUpdateInit();\n   `);
define('LIVEUPDATE_ANDROID_IMPL', `
    private void mfLiveUpdateInit() {
        LiveUpdateManager.getInstance(this).obtainConfiguration(false, new LiveUpdateListener());
    }\n`); 
define('LIVEUPDATE_ANDROID_LISTENERS', `   
    class LiveUpdateListener extends BaseListener implements ConfigurationListener {
        @Override
        public void onSuccess(final Configuration configuration) {
            {liveUpdateSchema}  
        }

        @Override
        public void onFailure(WLFailResponse wlFailResponse) {
            changeStatusMessage("Failed to obtain live update configuration");                                   
        }
    }\n`);
define('LIVEUPDATE_IOS_IMPORTS', `import IBMMobileFirstPlatformFoundationLiveUpdate\n`);
define('LIVEUPDATE_IOS_IMPL_CALL', `     mfLiveUpdateInit()\n   `);      
define('LIVEUPDATE_IOS_IMPL', `
    func mfLiveUpdateInit() {
        LiveUpdateManager.sharedInstance.obtainConfiguration(useCache:false, completionHandler: liveUpdateHandler)
    }`); 
define('LIVEUPDATE_IOS_HANDLER', `    
    func liveUpdateHandler(configuration: Configuration?, error: NSError?) {
        if error == nil {
          {liveUpdateSchema}
        } else {
          changeStatusText("Failed to obtain live update configuration")
        }
    }\n`);
define('LIVEUPDATE_IONIC_IMPL_CALL', `
    renderer.listenGlobal('document', 'mfpjsloaded', () => {
      this.initializeLiveUpdate();
    });\n    `);
define('LIVEUPDATE_IONIC_IMPL', `initializeLiveUpdate() {
    LiveUpdateManager.obtainConfiguration( { useClientCache: false }, function(configuration) {
        var liveUpdateText = 'LiveUpdate schema: ' + JSON.stringify({
          ...configuration.properties,
          ...configuration.features
        });
        var statusElement = <HTMLElement>(
          document.querySelector('[studio-id="home_Label_1150"]')
        );        
        statusElement.textContent = liveUpdateText;
      }, function(error) {
        var statusElement = <HTMLElement>(
          document.querySelector('[studio-id="home_Label_1150"]')
        );           
        statusElement.textContent = 'Failed to obtain live update configuration';
      }            
    );
  }\n  `);
define('PUSH_IONIC_IMPL_CALL', `  this.initializePush();\n    `);
define('PUSH_IONIC_IMPL', `
  initializePush() {
    if (this.platform.is("ios") || this.platform.is("android")) {
      MFPPush.initialize(
        function(successResponse) {
          MFPPush.registerNotificationsCallback(notificationReceived);
          WLAuthorizationManager.obtainAccessToken("push.mobileclient").then(
            function(accessToken) {
              MFPPush.registerDevice(null, successCallback, failureCallback);
            });
        },
        function(failureResponse) {
          var statusElement = <HTMLElement>(
            document.querySelector('[studio-id="home_Label_1150"]')
          );            
          statusElement.textContent = 'Failed to initialize Push';
        }
      );
    }
  }`);
define('PUSH_IONIC_STMTS', `
declare let MFPPush: any;
let notificationReceived = function(message) {
  var statusElement = <HTMLElement>(
    document.querySelector('[studio-id="home_Label_1150"]')
  );    
  if (message.alert.body !== undefined) {
    statusElement.textContent = 'Received notification: ' + message.alert.body;
  } else {
    statusElement.textContent = 'Received notification: ' + message.alert;
  }
};
let successCallback = function(response) {
  var statusElement = <HTMLElement>(
    document.querySelector('[studio-id="home_Label_1150"]')
  );            
  statusElement.textContent = 'Device successfully registered for push';    
};
let failureCallback = function(response) {
  var statusElement = <HTMLElement>(
    document.querySelector('[studio-id="home_Label_1150"]')
  );            
  statusElement.textContent = 'Failed to register the device for push'; 
};`);
define('ANALYTICS_IONIC_ENABLE', `
    renderer.listenGlobal('document', 'mfpjsloaded', () => {
      WL.Analytics.enable();
    });\n    `);
define('ANALYTICS_IONIC_LOG_CALL', `  this.mfAnalyticsInit();\n  `);
define('ANALYTICS_IONIC_LOG', `
  mfAnalyticsInit () {
    WL.Analytics.log('App started successfully...', '{APP_NAME}');
    WL.Analytics.send();

    var logger = WL.Logger.create({ pkg: '{APP_ID}' });
    logger.debug('App started successfully...');
    WL.Logger.send();
  }\n`);    
define('OCLOGGER', `
extension OCLogger {
    func logTraceWithMessages(message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_TRACE, message: message, args:getVaList(args), userInfo:Dictionary<String, String>())
    }
    func logDebugWithMessages(message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_DEBUG, message: message, args:getVaList(args), userInfo:Dictionary<String, String>())
    }
    func logInfoWithMessages(message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_INFO, message: message, args:getVaList(args), userInfo:Dictionary<String, String>())
    }
    func logWarnWithMessages(message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_WARN, message: message, args:getVaList(args), userInfo:Dictionary<String, String>())
    }
    func logErrorWithMessages(message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_ERROR, message: message, args:getVaList(args), userInfo:Dictionary<String, String>())
    }
    func logFatalWithMessages(message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_FATAL, message: message, args:getVaList(args), userInfo:Dictionary<String, String>())
    }
    func logAnalyticsWithMessages(message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_ANALYTICS, message: message, args:getVaList(args), userInfo:Dictionary<String, String>())
    }        
    func logTraceWithUserInfo(userInfo:Dictionary<String, String>, message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_TRACE, message: message, args:getVaList(args), userInfo:userInfo)
    }
    func logDebugWithUserInfo(userInfo:Dictionary<String, String>, message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_DEBUG, message: message, args:getVaList(args), userInfo:userInfo)
    }
    func logInfoWithUserInfo(userInfo:Dictionary<String, String>, message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_INFO, message: message, args:getVaList(args), userInfo:userInfo)
    }
    func logWarnWithUserInfo(userInfo:Dictionary<String, String>, message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_WARN, message: message, args:getVaList(args), userInfo:userInfo)
    }
    func logErrorWithUserInfo(userInfo:Dictionary<String, String>, message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_ERROR, message: message, args:getVaList(args), userInfo:userInfo)
    }
    func logFatalWithUserInfo(userInfo:Dictionary<String, String>, message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_FATAL, message: message, args:getVaList(args), userInfo:userInfo)
    }
    func logAnalyticsWithUserInfo(userInfo:Dictionary<String, String>, message:String, _ args: CVarArg...) {
        log(withLevel: OCLogger_ANALYTICS, message: message, args:getVaList(args), userInfo:userInfo)
    }
}`);
