/*
	Licensed Material - Property of IBM

	(C) Copyright 2015, 2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';

var moduleName = 'adapters';

var async = require('async');
var AdmZip = require('adm-zip');
var fs = require('fs');
var Q = require('q');
var path = require('path');
var rimraf = require('rimraf');

module.exports.getAdapters = function(serverDetails, mfpAdapterZipPath) {
  var deferred = Q.defer();
  
  var adapters = [];
  // get the adapters zip fom the MF server
  serverDetails.exportAdapters(mfpAdapterZipPath).then(function () {
    // Intermittently, the zip download on filesystem takes time, but flow
    // enters inside and breaks while creating admzip.
    // So wrapping it in setTimeout
    setTimeout(function() {
      //extract the adapters zip 
      var mfpAdapterZip = new AdmZip(mfpAdapterZipPath);
      mfpAdapterZip.extractAllTo(path.join(mfpAdapterZipPath, '..'), true, true);
      const adapterPackagePath =  path.join(mfpAdapterZipPath, '..', 'adapter', 'adapterPackage');
      // read the extracted adapters dir
      fs.readdir(adapterPackagePath, function(err, files) {
        if (err) {
          log.silly(moduleName, 'Failed to read extracted adapter folder. Reason: ' + err);
          deferred.resolve(adapters);
        } else {
          // for each adapter, copy the .adapter file to mfpconfig and 
          // get its configuration details from MF server
          async.forEachSeries(files, function (file, callback) {
            if (file.endsWith('.adapter')) {
              serverDetails.getAdapterConfig(path.basename(file, '.adapter')).then(function (response) {
                adapters.push(response);
                fs.copyFileSync(path.join(adapterPackagePath, file), path.join(mfpAdapterZipPath, '..', file));
                callback();
              }).catch(function (err){
                callback(err)
              })
            }
          }, function(err) {
            if (err) {
              log.silly(moduleName, 'Failed to get details of adapters. Reason: ' + err);
            }          
            rimraf.sync(path.join(mfpAdapterZipPath, '..', 'adapter'));
            rimraf.sync(mfpAdapterZipPath);
            deferred.resolve(adapters);          
          });
        }
      });   
    }, 2000);               
  }).catch(function (err) {
    deferred.resolve();
  });
  

  return deferred.promise;
}
