/*
	Licensed Material - Property of IBM

	(C) Copyright 2015, 2016 IBM Corp.

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/


'use strict';

var async = require('async');
var fs = require('fs');
var findup = require('findup-sync');
var log = require('npmlog');
var path = require('path');
var Q = require('q');
var utils = require('./requestUtils');

var moduleName = 'adapters';

module.exports.configureAdapters = function(appJson, serverInfo) {
  var deferred = Q.defer();   

  if (appJson.adapters) {
    const options = {
      path : '/mfpadmin/management-apis/2.0/runtimes/mfp/adapters',
      method : 'POST'
    }  
    const mfpConfigPath = findup('mfpconfig');
    async.forEachSeries(appJson.adapters, function (adapter, callback) {
      if (adapter.adapter) {          
        const payload = {
          'file' : fs.createReadStream(path.join(mfpConfigPath, adapter.adapter + ".adapter"))
        } 
        utils.sendFileRequest(serverInfo, options, payload).then(function () {
          var confOptions = {
            path : `/mfpadmin/management-apis/2.0/runtimes/mfp/adapters/${adapter.adapter}/config`,
            method : 'PUT'
          }             
          utils.sendRequest(serverInfo, confOptions, adapter).then(function () {
            callback();                      
          }).catch(function (err) {
            log.silly(moduleName, `Failed to configure adapter ${adapter.adapter}. Reason : ${err}`);
            callback(err); 
          });                                  
        }).catch(function (err) {
          log.silly(moduleName, `Failed to deploy adapter ${adapter.adapter}. Reason : ${err}`);
          callback(err);
        });                    
      } else {
        callback();
      }              
    }, function (err) {
      if (err) {
        deferred.reject(err);
      } else {
        deferred.resolve();
      }
    });
  } else {
    deferred.resolve();
  }   

  return deferred.promise;
}
